// Scene.cpp: implementation of the Scene class.
//
//////////////////////////////////////////////////////////////////////

#include "Scene.h"

int Scene::sirka,Scene::vyska;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Scene::Scene()
{
	pvb = NULL;
	texture = NULL;
	InitDX();
	if(fullscreen)ShowCursor(0);
	if(!CreateVertexBuffers()) { error=1; return; }

	pd->SetTextureStageState(0, D3DTSS_MINFILTER,D3DTEXF_LINEAR);
	pd->SetTextureStageState(0, D3DTSS_MAGFILTER,D3DTEXF_LINEAR);
//	pd->SetTextureStageState(0, D3DTSS_MIPFILTER,D3DTEXF_POINT);	// bilinear
	pd->SetTextureStageState(0, D3DTSS_MIPFILTER,D3DTEXF_LINEAR);	// trilinear
	
	if( FAILED(
	D3DXCreateTextureFromFileEx( pd, "data/textura.jpg", 0, 0, D3DX_DEFAULT, 0, D3DFMT_R8G8B8, D3DPOOL_DEFAULT, 
						D3DX_FILTER_BOX, D3DX_FILTER_BOX, 0xFF000000, NULL, NULL, &texture)
				))
	{texture = NULL; MessageBox(hWnd, "Textura Nenajdena", "Error", MB_OK); error=1; return;}
}

Scene::~Scene()
{
	if(pvb!=NULL) pvb->Release();
	if(texture!=NULL)texture->Release();
	if(fullscreen)ShowCursor(1);
}

void Scene::DrawDXScene()
{
	mat4 m;
    if( NULL == pd )return;

    // Clear the backbuffer to a blue color
	pd->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );

    // Begin the scene
    pd->BeginScene();

	if(keys['P'])pd->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME);
	else pd->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID);

	static float angle_x=0, angle_y=0, angle_z=0;
	angle_x+=0.2f;
	angle_y+=0.3f;
	angle_z+=0.4f;
	m.RotX(angle_x);
	m.RotY(angle_y);
	m.m34 = 5.f;

	pd->SetTransform( D3DTS_WORLD, (struct _D3DMATRIX *)(void*)&m );

	pd->SetTexture( 0, texture );
	pd->SetStreamSource( 0, pvb, sizeof(CUSTOMVERTEX) );
	pd->SetVertexShader( D3DFVF_CUSTOMVERTEX );
	for(int i=0; i<6; i++)
		pd->DrawPrimitive( D3DPT_TRIANGLESTRIP, 4*i, 2 );
	
	if(keys[VK_F12])SaveScreen_BMP();
	// End the scene
	pd->EndScene();

	// Present the backbuffer contents to the display
	pd->Present( NULL, NULL, NULL, NULL );
}

void Scene::ReSizeDXScene(int width, int height)
{
	sirka = width;
	vyska = height;
	
	D3DXMATRIX matProj;
	D3DXMatrixPerspectiveFovLH( &matProj, FIELD_OF_VIEW*PI180, (float)sirka/(float)vyska, Z_NEAR_VIEW_PLANE, Z_FAR_VIEW_PLANE );
//	D3DXMatrixOrthoOffCenterLH( &matProj, 0, 800, 0, 600, -1, 1 );
	pd->SetTransform( D3DTS_PROJECTION, &matProj );
}

void Scene::InitDX()
{
	if(!fullscreen)
	{
		RECT		WindowRect;					// miesto pre velkost okna
		WindowRect.left=(long)0;
		WindowRect.top=(long)0;
		GetClientRect(hWnd,&WindowRect);				//zistenie rozmerov uzivatelskej casti okna
		ReSizeDXScene(WindowRect.right,WindowRect.bottom);	//nastavenie perspektivy
	}
	else ReSizeDXScene(screen_x, screen_y);	//definovana v tomto subore

	pd->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE );
	
	// Enable z-buffering.
	pd->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
	pd->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESS);

	mat4 m;	
	pd->SetTransform( D3DTS_VIEW, (struct _D3DMATRIX *)(void*)&m );
	pd->SetTransform( D3DTS_WORLD, (struct _D3DMATRIX *)(void*)&m );
}

int Scene::CreateVertexBuffers()
{
	CUSTOMVERTEX v[24];		// 4*6

	int i=0;

	// Front Face
	v[i].t0.set(0.0f, 1.0f); v[i++].v.set(-1.0f, -1.0f,  1.0f);
	v[i].t0.set(1.0f, 1.0f); v[i++].v.set( 1.0f, -1.0f,  1.0f);
	v[i].t0.set(0.0f, 0.0f); v[i++].v.set(-1.0f,  1.0f,  1.0f);
	v[i].t0.set(1.0f, 0.0f); v[i++].v.set( 1.0f,  1.0f,  1.0f);
	
	// Back Face
	v[i].t0.set(1.0f, 1.0f); v[i++].v.set(-1.0f, -1.0f, -1.0f);
	v[i].t0.set(1.0f, 0.0f); v[i++].v.set(-1.0f,  1.0f, -1.0f);
	v[i].t0.set(0.0f, 1.0f); v[i++].v.set( 1.0f, -1.0f, -1.0f);
	v[i].t0.set(0.0f, 0.0f); v[i++].v.set( 1.0f,  1.0f, -1.0f);

	// Top Face
	v[i].t0.set(0.0f, 0.0f); v[i++].v.set(-1.0f,  1.0f, -1.0f);
	v[i].t0.set(0.0f, 1.0f); v[i++].v.set(-1.0f,  1.0f,  1.0f);
	v[i].t0.set(1.0f, 0.0f); v[i++].v.set( 1.0f,  1.0f, -1.0f);	
	v[i].t0.set(1.0f, 1.0f); v[i++].v.set( 1.0f,  1.0f,  1.0f);

	// Bottom Face
	v[i].t0.set(1.0f, 0.0f); v[i++].v.set(-1.0f, -1.0f, -1.0f);
	v[i].t0.set(0.0f, 0.0f); v[i++].v.set( 1.0f, -1.0f, -1.0f);
	v[i].t0.set(1.0f, 1.0f); v[i++].v.set(-1.0f, -1.0f,  1.0f);
	v[i].t0.set(0.0f, 1.0f); v[i++].v.set( 1.0f, -1.0f,  1.0f);

	// Right face
	v[i].t0.set(1.0f, 1.0f); v[i++].v.set( 1.0f, -1.0f, -1.0f);
	v[i].t0.set(1.0f, 0.0f); v[i++].v.set( 1.0f,  1.0f, -1.0f);
	v[i].t0.set(0.0f, 1.0f); v[i++].v.set( 1.0f, -1.0f,  1.0f);
	v[i].t0.set(0.0f, 0.0f); v[i++].v.set( 1.0f,  1.0f,  1.0f);
	
	// Left Face
	v[i].t0.set(0.0f, 1.0f); v[i++].v.set(-1.0f, -1.0f, -1.0f);
	v[i].t0.set(1.0f, 1.0f); v[i++].v.set(-1.0f, -1.0f,  1.0f);
	v[i].t0.set(0.0f, 0.0f); v[i++].v.set(-1.0f,  1.0f, -1.0f);
	v[i].t0.set(1.0f, 0.0f); v[i++].v.set(-1.0f,  1.0f,  1.0f);

	if( FAILED( pd->CreateVertexBuffer( 24*sizeof(CUSTOMVERTEX),0, 
								D3DFVF_CUSTOMVERTEX,D3DPOOL_DEFAULT, &pvb ) ) )return 0;
	void* p;
	if( FAILED( pvb->Lock( 0, 24*sizeof(CUSTOMVERTEX), (BYTE**)&p, 0 ) ) )return 0;
	memcpy( p, v, 24*sizeof(CUSTOMVERTEX) );
	pvb->Unlock();
	
	return 1;
}

void Scene::SaveScreen_BMP()
{
	int scr_size_x; int scr_size_y;
	IDirect3DSurface8 *sur;
	D3DDISPLAYMODE Mode;
	
	pd->GetDisplayMode( &Mode);
	Mode.Format = D3DFMT_A8R8G8B8;
	if(pd->CreateImageSurface(Mode.Width, Mode.Height, Mode.Format, &sur)!=D3D_OK )return;
	
	D3DSURFACE_DESC surd;
	if(pd->GetFrontBuffer( sur)!=D3D_OK){sur->Release(); return;}
	sur->GetDesc( &surd);

	D3DLOCKED_RECT LockedRect;
	if(sur->LockRect( &LockedRect, NULL, D3DLOCK_READONLY)!=D3D_OK)
	{
		sur->UnlockRect();
		sur->Release();
		return;
	}

	scr_size_x = surd.Width;
	scr_size_y = surd.Height;

	BITMAPFILEHEADER fileheader;
	BITMAPINFOHEADER infoheader;

	FILE *sub;
	int line_size = 3*scr_size_x;
	if(line_size%4)line_size += 4 - line_size%4;

	unsigned char* buf = new unsigned char[3*line_size*scr_size_y];
	char file[30];

	if(buf==NULL )
	{
		sur->UnlockRect();
		sur->Release();
		return;
	}

	int Pitch = LockedRect.Pitch;
	unsigned char* p = (unsigned char*) LockedRect.pBits;
	for(unsigned int y=0; y<surd.Height; y++)
	{
		for(unsigned int x=0; x<surd.Width; x++)
		{
			buf[line_size*y+3*x+0] = p[Pitch*(surd.Height-y-1)+4*x+0];
			buf[line_size*y+3*x+1] = p[Pitch*(surd.Height-y-1)+4*x+1];
			buf[line_size*y+3*x+2] = p[Pitch*(surd.Height-y-1)+4*x+2];
		}
	}
	sur->UnlockRect();
	sur->Release();

	for(int c=0;c<999;c++)
	{
		char pom[20];
		strcpy(file,"screen");
		if(c<100)strcat(file,"0");
		if(c<10)strcat(file,"0");
		strcat(file,itoa(c,pom,10));
		strcat(file,".bmp");
		sub = fopen(file,"r");
		if(sub==NULL)break;
		if(sub!=NULL)fclose(sub);
	}
	
	fileheader.bfType = 0x4D42; // Magic identifier   - "BM"	| identifikacia BMP suboru musi byt "BM"
	fileheader.bfSize = 3*scr_size_x*scr_size_y+sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);	// File size in bytes			| velkos suboru v byte
	fileheader.bfReserved1 = 0;
	fileheader.bfReserved2 = 0;
	fileheader.bfOffBits = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);	// Offset to image data, bytes	| posun na zaciatok dat
	
	infoheader.biSize = sizeof(BITMAPINFOHEADER);	// Header size in bytes			| velkost hlavicky BITMAPINFOHEADER
	infoheader.biWidth = scr_size_x;	// Width of image			| sirka obrazka - sizeX
	infoheader.biHeight = scr_size_y;	// Height of image			| vyska obrazka - sizeY
	infoheader.biPlanes = 1;		// Number of colour planes	| pocet farebnych rovin musi bit 1
	infoheader.biBitCount = 24;		// Bits per pixel			| bitov na pixel moze bit 1,4,8,24
	infoheader.biCompression = 0;	// Compression type			| typ compresie , 0 - bez kompresie
	infoheader.biSizeImage = line_size*infoheader.biHeight ;	// Image size in bytes		| velkost obrazka v byte
	infoheader.biXPelsPerMeter = 0;	// Pixels per meter X		| pixelov na meter v smere x
	infoheader.biYPelsPerMeter = 0;	// Pixels per meter Y		| pixelov na meter v smere y
	infoheader.biClrUsed = 0;		// Number of colours		| pocet  farieb v palete, ak 0 vsetky su pouzivane
	infoheader.biClrImportant = 0;	// Important colours		| dolezite farby v palete, ak 0 vsetky su dolezite
	
	sub = fopen(file,"wb");
	fwrite( &fileheader, sizeof(BITMAPFILEHEADER), 1, sub);
	fwrite( &infoheader, sizeof(BITMAPINFOHEADER), 1, sub);
	fwrite( buf, line_size*scr_size_y, 1, sub);

	delete[] buf;
	fclose(sub);
}
